#!/usr/bin/env python
# -*- coding: utf-8 -*-

from builtins import str
import time

from db_paribhasha import credentials
from db_paribhasha.dynamo import dynamo_helpers as db_helpers
from db_paribhasha.dynamo.engine import Engine
from db_paribhasha.error_handler import OperationNotPerformedError


def create_table():
    """
    Creates table if table doesn't exist
    """

    if Engine.exists():
        raise OperationNotPerformedError("Table already exists")
    Engine.create_table(
        read_capacity_units=credentials.DYNAMO_GENERIC_RW_CAP,
        write_capacity_units=credentials.DYNAMO_GENERIC_RW_CAP)


def create_engine_request(parent_id, engine_data, engine_type, parent_type,
                          user_id):
    """
    Creates an engine request

    Parameters
    ----------
    parent_id : String/Int
        Run/WF ID
    engine_data : Dict
        Dict containing required engine data
    engine_type : String
        Type of engine to run
    parent_type : String
        Parent type: Run or WF or project
    user_id : Int/String
        Unique ID of the user

    Returns
    -------
    calculated_id
        Unique ID for the engine request
    """

    created_ts = int(round(time.time() * 1000))
    calculated_id = engine_type + '-' + str(parent_id) + '-' + str(created_ts)
    engine_request = Engine(
        parent_id=int(parent_id),
        created_ts=created_ts,
        calculated_id=calculated_id,
        modified_ts=created_ts,
        engine_data=engine_data,
        user_id=user_id,
        parent_type=parent_type,
        engine_type=engine_type,
        state="init")
    db_helpers.save_dynamo_object(engine_request)
    return calculated_id


def get_engine_request_detail(engine_request_id, user_id):
    """
    Returns the engine request pynamo object

    Parameters
    ----------
    engine_request_id : String
        Unique engine request ID

    user_id: Integer
        User ID

    Returns
    -------
    engine request
        pynamo object
    """
    engine_request = Engine.get(int(user_id), engine_request_id)
    return engine_request
