from builtins import str
from db_paribhasha.sentry_client import sentry_client
from db_paribhasha import consts_errs as cnsts


class BaseExceptionError(Exception):
    """
    Base class for custom Mithoo Microservices exceptions.
    """
    status_code = 500
    message = cnsts.BASE_ERROR
    error_type = cnsts.err_BaseExceptionError_code

    def __init__(self, message=None, error_type=None, status_code=None, payload=None):
        """
        Constructor for the BaseExceptionError class.

        Logs the error and debug info to Sentry dashboard.


        Parameters
        ----------
        message: String
            Message for the error raised
        status_code: Integer
            Status code for the error raised
        payload: String/Dict
            Error message payload which can be used to append useful information
            with the error.
        """
        Exception.__init__(self)
        if message:
            self.message = str(message)
        if error_type:
            self.error_type = error_type
        if status_code:
            self.status_code = status_code
        self.payload = payload
        if not message:
            self.message = cnsts.BASE_ERROR
        if message is None:
            message = cnsts.SOME_ERROR
        sentry_client.log_error(self.message)
        sentry_client.logall_debug_queue()

    def to_dict(self):
        rv = dict(self.payload or ())
        rv['messages'] = self.message
        return rv

    def __str__(self):
        return self.message


class DatabaseEntryError(BaseExceptionError):
    message = cnsts.DATABASEENTRYERROR
    error_type = cnsts.err_DatabaseEntryError_code

    def __init__(self, message=None, error_type=None):
        if message:
            self.message = message
        if error_type:
            self.error_type = error_type


class InvalidInputError(BaseExceptionError):
    message = cnsts.INVALID_INPUT
    error_type = cnsts.err_InvalidInputError_code

    def __init__(self, message=None, error_type=None):
        if message:
            self.message = message
        if error_type:
            self.error_type = error_type


class DynamoDBError(BaseExceptionError):
    def __init__(self, message=None):
        if not message:
            message = cnsts.DYNAMO_DB_OP_ERR
        BaseExceptionError.__init__(self, message)


class OperationNotPerformedError(BaseExceptionError):
    def __init__(self, message=None):
        if not message:
            message = cnsts.NOT_PERFORMED
        BaseExceptionError.__init__(self, message)
