from db_paribhasha.error_handler import DatabaseEntryError
from db_paribhasha.base_models.sql_base_model import session
from db_paribhasha.workflow.workflow_request import workflow_request_table
from db_paribhasha import db_constants as db_const

def get_value(workflow_req_id, column_name):
    """
    Returns value of a given cell in database.

    Parameters
    ------------
    workflow_req_id: Integer
        Primary key of the workflow_request table
    coloum_name: String
        Name of the coloumn in the datbase

    Returns:
    -------------
    column_value : str | int
        Value of the cell in database
    """
    if column_name not in list(workflow_request_table.WorkflowRequest.__table__.\
            columns.keys()):
        raise DatabaseEntryError(db_const.INVALID_COLUMN)

    if has_workflow_request(workflow_req_id):
        workflow_request_record = session.query(workflow_request_table.WorkflowRequest).get(workflow_req_id)
        return getattr(workflow_request_record, column_name)
    else:
        raise DatabaseEntryError(db_const.INVALID_RUN_ID)


def has_workflow_request(workflow_req_id):
    """
    Checks if a given workflow_request is present in the database

    Parameters
    ----------
    workflow_req_id: Integer
        Workflow request id to be checked

    Returns
    -------
    Boolean
        If the workflow request exists or not
    """

    workflow_request_record = session.query(workflow_request_table.WorkflowRequest).\
        get(workflow_req_id)
    if workflow_request_record is not None:
        return True
    else:
        return False


def is_state_uploaded(workflow_req_id):
    """
    Checks if a given workflow_request is in the uploaded state

    Parameters
    ----------
    workflow_req_id: Integer
        Workflow request id to be checked

    Returns
    -------
    Boolean
        If the workflow request is in uploaded state or not
    """

    value = get_value(workflow_req_id, 'state')
    if value == db_const.STATE_WORKFLOW_REQ_UPLOADED:
        return True
    else:
        return False


def get_workflow_requests(project_id):
    '''
    To get all the workflow requests which belongs to project with given
    project id

    Parameters
    -----------
    project_id: int 
        project id
    
    Returns
    --------
    list:
        list of wokflow requests belonging to current project_id
    '''
    wf_requests = session.query(workflow_request_table.WorkflowRequest).filter_by(project=project_id).all()
    
    return [req.as_dict() for req in wf_requests]


def get_workflow_request(where_args):
    '''
    To get the workflow request which satisfying the constraints given in 
    'where_args' in argument.

    Parameters
    -----------
    where_args:dict
        dictionary containing the where arguments of the sql query
    
    Returns
    --------
    dict|None:
        dictionary containing the wokflow request details.
        It returns None is no such workflow request found
    '''
    wf_request = session.query(workflow_request_table.WorkflowRequest).filter_by(**where_args).first()
    if wf_request:
        return wf_request.as_dict()
    return None
